#------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------

import sys
import commands
import exceptions
from cli.utils import load_config
from cli.utils import checks
from cli.utils import server_errors

class ListImageError(exceptions.Exception):
    pass

def print_list_header():
    '''Print List Header'''
    print 'Image Details \n'

def change_strings(msg_info, msg_header):
    if len(msg_info) < len(msg_header):
        a = len(msg_header) - len(msg_info)
        msg_info += ' '*a
    else:
        a = len(msg_info) - len(msg_header)
        msg_header += ' '*a
    return msg_info, msg_header

def print_image_details(info):
    '''Print Image details'''
    if type(info) == dict:
        msg_h = 'hostname'
        msg_i = info['hostname']
        msg_i, msg_h = change_strings(msg_i, msg_h)
        msg_i += '  ' + info['state']
        msg_h += '  status'
        msg_i, msg_h = change_strings(msg_i, msg_h)
        msg_i += '  ' + info['architecture']
        msg_h += '  arch'
        msg_i, msg_h = change_strings(msg_i, msg_h)
        msg_i += '  ' + info['memory']
        msg_h += '  memory'
        msg_i, msg_h = change_strings(msg_i, msg_h)
        msg_i += '  ' + info['cores']
        msg_h += '  cores'
        msg_i, msg_h = change_strings(msg_i, msg_h)
        msg_i += '  ' + info['speed']
        msg_h += '  speed'
        print msg_h
        print msg_i

class ListImages(object):
    def __init__(self, user_data = {}, image_location = '', path = '/-/'):
        self.image_location = image_location
        self.path = path
        self.cmd_name = 'curl'
        self.user_data = user_data

    def get_command(self):
        '''Gets command string'''

        not_show_progress_meter = ' -s '

        info_option = ' -i '

        map_value = load_config.UserConfigCommand().get_map()

        cert,key = user.set_cert_key(data=self.user_data)

        cert_option = (map_value['user_cert'] + ' ' +
            cert + ' ')
        key_option = (map_value['user_key'] + ' ' +
            key + ' ')

        ca_option = (map_value['ca'] + ' ' +
            self.user_data['user']['ca'] + ' ')

        content_type_option = ("-H 'Accept: " +
           self.user_data['content']['type'] + "' ")

        url = (' https://' +
           self.user_data['server']['endpoint'] + ':' +
           self.user_data['server']['port'] + '/')

        cmd = (self.cmd_name + not_show_progress_meter + info_option +
           ca_option + cert_option + key_option +
           content_type_option)

        if self.image_location == '':
            cmd += url + self.path
        else:
            cmd += self.image_location 

        return cmd

    def set_cmd_name(self):
        self.cmd_name = checks.get_cmd_name(self.cmd_name)

    def run_command(self):
        '''Runs command'''
        self.set_cmd_name()
        cmd_output = commands.getstatusoutput(self.get_command())
        return cmd_output

    def get_output(self):
        '''Returns Images Information'''
        output = self.run_command()

        correct_values = []
        msg = ''

        if len(output) > 0 and output[0] == 0:
            values = output[1].split('\n')

            errors = server_errors.ServerErrors()
            for x in values:
                if x in errors.get_errors():
                    msg = x

            if msg == '':
                for y in errors.get_occi_errors():
                    if y in values[0]:
                        raise ListImageError(values[0].split(y)[1])
                for x in values[5:]:
                    correct_values.append(str(x))

        elif output[0] > 0:
            raise ListImageError(output[1][1:])    

        return correct_values, msg

class ListImage(ListImages):
    def __init__(self, user_data, image_location):
        super(ListImage, self).__init__(user_data=user_data, image_location=image_location)

    def get_output(self):
        '''Returns Image Information'''

        values, msg = ListImages.get_output(self)

        return values, msg
